LANGUAGES_NL = {
    "Albanees": "sq-AL",
    "Amhaars": "am-ET",
    "Arabisch": "ar-SA",
    "Armeens": "hy-AM",
    "Azerbeidzjaans": "az-AZ",
    "Baskisch": "eu-ES",
    "Bengaals": "bn-BD",
    "Bhutanees": "dz-BT",
    "Birmaans": "my-MM",
    "Bosnisch": "bs-BA",
    "Bretons": "br-FR",
    "Bulgaars": "bg-BG",
    "Catalaans": "ca-ES",
    "Chinees": "zh-CN",  # Vereenvoudigd Chinees, voor Traditioneel Chinees zou het "zh-TW" zijn
    "Corsicaans": "co-FR",
    "Deens": "da-DK",
    "Duits": "de-DE",
    "Dzongkha": "dz-BT",
    "Engels": "en-GB",
    "Engels (Brits)": "en-GB",
    "Engels (VS)": "en-US",
    "Amerikaans": "en-US",
    "Estlands": "et-EE",
    "Farsi": "fa-IR",
    "Perzisch": "fa-IR",
    "Fijisch": "fj-FJ",
    "Fins": "fi-FI",
    "Frans": "fr-FR",
    "Gaelisch (Schots)": "gd-GB",
    "Galicisch": "gl-ES",
    "Georgisch": "ka-GE",
    "Grieks": "el-GR",
    "Hebreeuws": "he-IL",
    "Hindi": "hi-IN",
    "Hongaars": "hu-HU",
    "IJslands": "is-IS",
    "Indonesisch": "id-ID",
    "Italiaans": "it-IT",
    "Japans": "ja-JP",
    "Javaans": "jv-ID",
    "Kazachs": "kk-KZ",
    "Khmer": "km-KH",
    "Koreaans": "ko-KR",
    "Kroatisch": "hr-HR",
    "Lao": "lo-LA",
    "Lets": "lv-LV",
    "Lithouws": "lt-LT",
    "Macedonisch": "mk-MK",
    "Malagasi": "mg-MG",
    "Maleis": "ms-MY",
    "Maltees": "mt-MT",
    "Maori": "mi-NZ",
    "Marathi": "mr-IN",
    "Mongools": "mn-MN",
    "Nederlands": "nl-NL",
    "Nepali": "ne-NP",
    "Noors": "no-NO",
    "Oekraïens": "uk-UA",
    "Oezbeeks": "uz-UZ",
    "Pools": "pl-PL",
    "Portugees": "pt-PT",
    "Portugees (Brazilië)": "pt-BR",
    "Braziliaans": "pt-BR",
    "Roemeens": "ro-RO",
    "Russisch": "ru-RU",
    "Servisch": "sr-RS",
    "Sloveens": "sl-SI",
    "Spaans": "es-ES",
    "Swahili": "sw-KE",
    "Zweeds": "sv-SE",
    "Thai": "th-TH",
    "Tsjechisch": "cs-CZ",
    "Turks": "tr-TR",
    "Vietnamees": "vi-VN",
    "Wit-Russisch": "be-BY",
    "Zulu": "zu-ZA"
}

LANGUAGES_EN = language_tags_full = {
    "Albanian": "sq-AL",
    "Amharic": "am-ET",
    "Arabic": "ar-SA",
    "Armenian": "hy-AM",
    "Azerbaijani": "az-AZ",
    "Basque": "eu-ES",
    "Bengali": "bn-BD",
    "Bhutanese": "dz-BT",
    "Burmese": "my-MM",
    "Bosnian": "bs-BA",
    "Breton": "br-FR",
    "Bulgarian": "bg-BG",
    "Catalan": "ca-ES",
    "Chinese": "zh-CN",  # Simplified Chinese, for Traditional Chinese it would be "zh-TW"
    "Corsican": "co-FR",
    "Danish": "da-DK",
    "German": "de-DE",
    "Dzongkha": "dz-BT",
    "English": "en-GB",
    "British English": "en-GB",
    "American English": "en-US",
    "American": "en-US",
    "English (US)": "en-US",
    "Estonian": "et-EE",
    "Persian": "fa-IR",
    "Fijian": "fj-FJ",
    "Finnish": "fi-FI",
    "French": "fr-FR",
    "Scottish Gaelic": "gd-GB",
    "Galician": "gl-ES",
    "Georgian": "ka-GE",
    "Greek": "el-GR",
    "Hebrew": "he-IL",
    "Hindi": "hi-IN",
    "Hungarian": "hu-HU",
    "Icelandic": "is-IS",
    "Indonesian": "id-ID",
    "Italian": "it-IT",
    "Japanese": "ja-JP",
    "Javanese": "jv-ID",
    "Kazakh": "kk-KZ",
    "Khmer": "km-KH",
    "Korean": "ko-KR",
    "Croatian": "hr-HR",
    "Lao": "lo-LA",
    "Latvian": "lv-LV",
    "Lithuanian": "lt-LT",
    "Macedonian": "mk-MK",
    "Malagasy": "mg-MG",
    "Malay": "ms-MY",
    "Maltese": "mt-MT",
    "Maori": "mi-NZ",
    "Marathi": "mr-IN",
    "Mongolian": "mn-MN",
    "Dutch": "nl-NL",
    "Nepali": "ne-NP",
    "Norwegian": "no-NO",
    "Ukrainian": "uk-UA",
    "Uzbek": "uz-UZ",
    "Polish": "pl-PL",
    "Portuguese": "pt-PT",
    "Brazilian Portuguese": "pt-BR",
    "Brazilian": "pt-BR",
    "Portuguese (Brazil)": "pt-BR",
    "Romanian": "ro-RO",
    "Russian": "ru-RU",
    "Serbian": "sr-RS",
    "Slovenian": "sl-SI",
    "Spanish": "es-ES",
    "Swahili": "sw-KE",
    "Swedish": "sv-SE",
    "Thai": "th-TH",
    "Czech": "cs-CZ",
    "Turkish": "tr-TR",
    "Vietnamese": "vi-VN",
    "Belarusian": "be-BY",
    "Zulu": "zu-ZA"
}

LANGUAGES = LANGUAGES_NL | LANGUAGES_EN
